#!/bin/bash

CLUSTER_USER="hacluster"
DEFAULT_CLUSTER_PWD="UFMclusterHA"
CLUSTER_NAME="ufmcluster"
CLUSTER_GROUP_NAME="ufmcluster-grp"
DRBD_PACEMAKER_RESOURCE_SUFFIX="drbd"
FSTAB_FILE=/etc/fstab
DRBD_RESOURCE=
DRBD_LOCATION=
DRBD_DISK=
DRBD_PORT=
DRBD_DEVICE=
SUDO=
MASTER_NODE_IP=
STANDBY_NODE_IP=
CURRENT_NODE_IP=
HACLUSTER_PWD=
VIRTUAL_IP=
PRIMARY_IP_TYPE=
SECONDARY_IP_TYPE=
ALL_PCS_RESOURCES=
FILE_SYSTEM_RESOURCES=()
DISK_STATE=()
DRBD_ROLE=()
DRBD_CONNECTIVITY=
PCS_VERSION=
PCS_PRE_0_10_VERSION="0.10.0"
PCS_PRE_0_10_8_VERSION="0.10.8"
PCS_PRE_0_10_1_VERSION="0.10.1"
PCS_PRE_0_11_0_VERSION="0.11.0"


MASTER_SCORE=50
STICKINESS=100
VIRTUAL_IP_RSRC_NAME="cluster_virtual_ip"
BACKUP_DIR="/ha_backup"
CLEAN_UP_BACKUP_DIR="/cleanup_backup"
NODE_ROLE_MASTER="master"
NODE_ROLE_STANDBY="standby"
ALL_CLUSTER_NODES=()
multipathd_services=("multipathd.socket" "multipathd.service")
multi_conf_flag="devnode \"\^drbd\[0-9\]\""
multipath_conf="blacklist { \n devnode "^drbd[0-9]" \n }"

VALID_DRBD_CONNECTIVITIES=("WFConnection" "Connected")

DEFAULT_PM_PORT=2224

echoerr() { echo "$@" 1>&2; }

print_title() {
    echo "*****"
    echo "$1"
    echo "*****"
}

declare -A ini_config_arr
parse_ini() {
    local config_file=$1
    while IFS='=' read -r ini_key ini_val || [ -n "$ini_key" ]; do
        ini_key=$(echo "$ini_key" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')
        ini_val=$(echo "$ini_val" | sed -e 's/^[[:space:]]*//' -e 's/[[:space:]]*$//')
        if [[ "$ini_key" =~ ^\#.*$ ]] || [[ "$ini_key" == "" ]]; then
            continue
        fi
        if [[ "$ini_key" =~ ^\[.*\]$ ]]; then
            section=$(echo $ini_key | tr -d '[]')
        elif [ -n "$ini_val" ]; then
            full_name=$(echo ${section}_${ini_key})
            ini_config_arr[$full_name]=$ini_val
        fi
    done < $config_file
}


get_pcs_version(){
    PCS_VERSION=$(pcs --version)
}

function verlte() {
    [  "$1" = "$(echo -e "$1\n$2" | sort -V | head -n1)" ]
}

authenticate_cluster_nodes(){
    local node1_ip="$1"
    local node2_ip="$2"
    if verlte ${PCS_VERSION} ${PCS_PRE_0_10_VERSION}; then
        ${SUDO}pcs cluster auth -u ${CLUSTER_USER} -p ${HACLUSTER_PWD} ${node1_ip}  ${node2_ip}
    else
        ${SUDO}pcs host auth -u ${CLUSTER_USER} -p ${HACLUSTER_PWD} ${node1_ip} addr=${node1_ip} ${node2_ip} addr=${node2_ip}
    fi
    return $?
}

setup_cluster_name(){
    local nodes_ips="$1"
    local ip_type=""
    if [ "${PRIMARY_IP_TYPE}" == "${IP_TYPE_V6}" ]; then
      local ip_type="--ipv6"
    fi

    if verlte ${PCS_VERSION} ${PCS_PRE_0_10_VERSION}; then
            ${SUDO}pcs cluster setup ${ip_type} --name ${CLUSTER_NAME} ${nodes_ips} --force
    else
        ${SUDO}pcs cluster setup ${CLUSTER_NAME} ${nodes_ips} --force
    fi
    return $?
}

_get_cluster_setup_nodes(){
    auth_node1=$1
    auth_node2=$2
    local nodes_ips=
    if [[ -n "${LOCAL_SECONDARY_IP}" && -n "${PEER_SECONDARY_IP}" ]]; then
      if verlte ${PCS_VERSION} ${PCS_PRE_0_10_VERSION}; then
        nodes_ips="${auth_node1},${LOCAL_SECONDARY_IP} ${auth_node2},${PEER_SECONDARY_IP}"
      else
        nodes_ips="${auth_node1} addr=${auth_node1} addr=${LOCAL_SECONDARY_IP} ${auth_node2} addr=${auth_node2} addr=${PEER_SECONDARY_IP}"
      fi
    else
      nodes_ips="${auth_node1} ${auth_node2}"
    fi
    echo "$nodes_ips"
}

init_cluster() {
    #authenticate cluster on both node
    print_title "authenticate cluster nodes"
    if [ "${PRIMARY_IP_TYPE}" == "${IP_TYPE_V6}" ] && verlte ${PCS_VERSION} ${PCS_PRE_0_10_VERSION}; then
        local_sync_hostname=$(get_hostname_from_ip ${LOCAL_SYNC_IP})
        peer_sync_hostname=$(get_hostname_from_ip ${PEER_SYNC_IP})
        local auth_node1=${local_sync_hostname}
        local auth_node2=${peer_sync_hostname}
        local nodes_ips=$(_get_cluster_setup_nodes ${auth_node1} ${auth_node2})
    else
        local auth_node1=${LOCAL_SYNC_IP}
        local auth_node2=${PEER_SYNC_IP}
        local nodes_ips=$(_get_cluster_setup_nodes ${auth_node1} ${auth_node2})
    fi
    authenticate_cluster_nodes "${auth_node1}" "${auth_node2}"
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    #setup cluster
    print_title "setup cluster"
    setup_cluster_name "${nodes_ips}"
    return $?
}

is_master_role() {
  if [ "${NODE_ROLE}" == "${NODE_ROLE_MASTER}" ]; then
    echo true
  else
    echo false
  fi
}

get_master_from_cluster() {
    local master_node=
    maintenance_mode=$(check_maintenance_mode)
    if [[ "$maintenance_mode" = 'true' ]]; then
      master_node=$(pcs resource 2> /dev/null | grep -oP '(?<=Master\s)\S+')
    else
      master_node=$(pcs status 2> /dev/null  | grep -E '(Masters|Promoted)' | awk -F: '{print $2}' | awk '{print $2}')
    fi
    echo "${master_node}"
}

get_hostname_for_ip_in_cluster() {
  ip=$1
  node_hostname=$(pcs status pcsd | grep ${ip} | awk -F":" '{print $1;}' | awk '{print $1;}')
  echo ${node_hostname}
}

start_cluster() {
    print_title "start cluster"

    ${SUDO}pcs cluster start --all
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    ${SUDO}pcs cluster enable --all
    return $?
}

show_cluster_status() {
    ${SUDO}pcs cluster status
    return $?
}

destroy_cluster(){
    echo "destroy pcs cluster"
    ${SUDO}pcs cluster destroy
    return $?
}

cleanup_pacemaker() {
    destroy_cluster
    return $?
}

cluster_utils_set_password() {
    #echo "${CLUSTER_USER}:${HACLUSTER_PWD}" | ${SUDO}chpasswd
    echo -e "${HACLUSTER_PWD}\n${HACLUSTER_PWD}" | ${SUDO}passwd $CLUSTER_USER
}

stop_drbd_service(){
    echo "stop drbd service"
    ${SUDO}systemctl stop drbd
    return $?
}

cluster_utils_pre_config() {
  #check if pcs is installed
  ${SUDO}pcs --version > /dev/null 2>&1
  if [ $? -ne 0 ]; then
      echoerr "pacemaker is not installed on this machine"
      echoerr "please install HA required packages before ha configuration"
      return 1
  fi
  #check if drbd is installed
  ${SUDO}drbdadm --version > /dev/null 2>&1
  if [ $? -ne 0 ]; then
      echoerr "drbd is not installed on this machine"
      echoerr "please install HA required packages before ha configuration"
      return 1
  fi
  #check if pcsd service is active
  ${SUDO}systemctl is-active pcsd > /dev/null 2>&1
  if [ $? -ne 0 ]; then
      echo "pcsd service is not active, starting pcsd service"
      ${SUDO}systemctl start pcsd

  else
    echo "pcsd service is running"
  fi
  #cleanup previous cluster configuration, just in case.
  ${SUDO}pcs cluster destroy > /dev/null 2>&1

  #stop drbd service
  stop_drbd_service

  #enable pre-requisite services
  ${SUDO}systemctl enable pcsd

  #disable services controlled by pacemaker
  systemd_services=(${ini_config_arr[pacemaker_systemd_services]})
  for srv in "${systemd_services[@]}"; do
    ${SUDO}systemctl is-active ${srv} > /dev/null 2>&1
    if [ $? -eq 0 ]; then
      echo "service ${srv} is running, you have to stop it before adding it to HA"
      echo "use: ${SUDO}systemctl stop ${srv}"
      return 1
    fi
    ${SUDO}systemctl disable ${srv}
  done
  # workaround for corosync issue
  ${SUDO}sed -i 's/^127.0.1.1/#127.0.1.1/g' /etc/hosts
  return 0
}

add_systemd_resource() {
    service_name=$1
    print_title "add systemd service: ${service_name}"
    #create service resource
    ${SUDO}pcs resource create ${service_name} systemd:${service_name} \
        op monitor interval=10s start interval=0 timeout='2min' stop interval=0 timeout='1min' \
        on-fail='restart' --group ${CLUSTER_GROUP_NAME} --disable
    return $?
}

add_virtual_ip_resource() {
    RESOURCE_NAME=${VIRTUAL_IP_RSRC_NAME}
    #create virtual IP resource
    print_title "create virtual IP resource"
    ${SUDO}pcs resource create ${RESOURCE_NAME} ocf:heartbeat:IPaddr2 ip=${VIRTUAL_IP} \
        op monitor interval=20s start interval=0s timeout=20s stop interval=0s timeout=20s \
        on-fail='restart' --group ${CLUSTER_GROUP_NAME} --disable
    return $?
}

_get_pcs_resources() {
  get_pcs_version
  get_drbd_resource_name
  local fs_resource="${DRBD_RESOURCE}_file_system"
   if verlte ${PCS_VERSION} ${PCS_PRE_0_10_VERSION}; then
       ALL_PCS_RESOURCES=($(${SUDO}pcs resource show --groups | grep ${CLUSTER_GROUP_NAME} | awk -F": " '{print $2}'))
   else
       ALL_PCS_RESOURCES=($(${SUDO}pcs resource group list | grep ${CLUSTER_GROUP_NAME} | awk -F": " '{print $2}'))
   fi
  ALL_PCS_RESOURCES=(${ALL_PCS_RESOURCES[@]/$fs_resource})
}

cluster_utils_failover() {
  get_pcs_version
  res=$(is_master)
  if [[ ${res} != true ]]; then
    echo "can not perform failover on slave node."
    return 1
  fi
  check_drbd_uptodate
  exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
  check_drbd_connected
  exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
  ${SUDO}pcs resource cleanup > /dev/null
  read_current_node_ip 'quiet'
  local this_node=${CURRENT_NODE_IP}
  local ip_configured=true
  if [ -z ${CURRENT_NODE_IP} ]; then
    this_node=$(hostname)
    ip_configured=false
  else
    PRIMARY_IP_TYPE=$(validate_ip ${CURRENT_NODE_IP})
    if [ "${PRIMARY_IP_TYPE}" == "${IP_TYPE_V6}" ]; then
      this_node=$(get_hostname_from_ip ${CURRENT_NODE_IP})
    fi
  fi
  local peer_node_hostname=
  # This returns IPv4 or IPv6 hostnames depends on the used IP version
  ALL_CLUSTER_NODES=($(pcs cluster corosync | grep ring0_addr | awk -F": " '{print $2}'))
  for node in "${ALL_CLUSTER_NODES[@]}"; do
    if [[ "${this_node}" != "${node}"* ]]; then
      if [ $ip_configured = false ]; then
        peer_node_hostname=${node}
      else 
        if verlte ${PCS_VERSION} ${PCS_PRE_0_10_VERSION}; then
          if [ "${PRIMARY_IP_TYPE}" == "${IP_TYPE_V6}" ]; then
            # pcs version < 0.10.0 and IPv6
            peer_node_hostname=${node}
          else
            # pcs version < 0.10.0 and IPv4
            peer_node_hostname=$(get_hostname_for_ip_in_cluster ${node})
          fi
        else
            peer_node_hostname=${node}
        fi
      fi
      break
    fi
  done
  if [ -z "${peer_node_hostname}" ]; then
      echoerr "Error: cannot resolve peer node hostname!"
      return 1
  fi
  ${SUDO}pcs resource clear ${CLUSTER_GROUP_NAME}
  # This commands requires hostname
  if verlte ${PCS_VERSION} ${PCS_PRE_0_11_0_VERSION}; then
    ${SUDO}pcs resource move ${CLUSTER_GROUP_NAME} ${peer_node_hostname} lifetime=PT15M
  else
    ${SUDO}pcs resource move-with-constraint ${CLUSTER_GROUP_NAME} ${peer_node_hostname} lifetime=PT15M
  fi
  return $?
}

cluster_utils_takeover() {
  get_pcs_version
  res=$(is_master)
  if [[ ${res} == true ]]; then
    echo "can not perform takeover on master node."
    return 1
  fi
  check_drbd_uptodate
  exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
  check_drbd_connected
  exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
  ${SUDO}pcs resource cleanup  > /dev/null
  read_current_node_ip 'quiet'
  local this_node_hostname=
  if [ -z ${CURRENT_NODE_IP} ]; then
    this_node_hostname=$(hostname)
  else
    PRIMARY_IP_TYPE=$(validate_ip ${CURRENT_NODE_IP})
    if [ "${PRIMARY_IP_TYPE}" == "${IP_TYPE_V6}" ]; then
      this_node_hostname=$(get_hostname_from_ip ${CURRENT_NODE_IP})
    else
      this_node_hostname=$(get_hostname_for_ip_in_cluster ${CURRENT_NODE_IP})
    fi
  fi
  ${SUDO}pcs resource clear ${CLUSTER_GROUP_NAME}
  # This commands requires hostname
  if verlte ${PCS_VERSION} ${PCS_PRE_0_11_0_VERSION}; then
    ${SUDO}pcs resource move ${CLUSTER_GROUP_NAME} ${this_node_hostname} lifetime=PT15M
  else
    ${SUDO}pcs resource move-with-constraint ${CLUSTER_GROUP_NAME} ${this_node_hostname} lifetime=PT15M
  fi
  return $?
}

add_drbd_pcs_resourses() {
  echo "Creating drbd pacemaker resources"
  add_drbd_pcs_resourse
  return $?
}

clone_drbd_resourse(){
    local drbd_resource=$1
    local drbd_master_resource=$2

    if verlte ${PCS_VERSION} ${PCS_PRE_0_10_VERSION}; then
        ${SUDO}pcs -f drbd_cfg resource master "${drbd_master_resource}" "${drbd_resource}" master-max=1 master-node-max=1 clone-max=2 clone-node-max=1 notify=true
    else
        ${SUDO}pcs -f drbd_cfg resource promotable "${drbd_resource}" promoted-max=1 promoted-node-max=1 clone-max=2 clone-node-max=1 notify=true
    fi
    return $?
}

add_drbd_pcs_resourse() {
    local drbd_resource="${DRBD_RESOURCE}_drbd"
    local drbd_master_resource=
    local fs_resource="${DRBD_RESOURCE}_file_system"
    local FORCE=

    if verlte ${PCS_VERSION} ${PCS_PRE_0_10_VERSION}; then
        drbd_master_resource="${drbd_resource}_master"
    else
        drbd_master_resource="${drbd_resource}-clone"
        FORCE="--force"
    fi

    ${SUDO}pcs cluster cib drbd_cfg
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    ${SUDO}pcs -f drbd_cfg resource create "${drbd_resource}" ocf:linbit:drbd drbd_resource=${DRBD_RESOURCE} op monitor interval=60s --group ${CLUSTER_GROUP_NAME} ${FORCE}
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    clone_drbd_resourse ${drbd_resource} ${drbd_master_resource}
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    ${SUDO}pcs cluster cib-push --config drbd_cfg
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    ${SUDO}pcs cluster cib fs_cfg
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    ${SUDO}pcs -f fs_cfg resource create "${fs_resource}" Filesystem device=${DRBD_DEVICE} directory=${DRBD_LOCATION} fstype="ext4" --group ${CLUSTER_GROUP_NAME}
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    ${SUDO}pcs  -f fs_cfg constraint colocation add "${fs_resource}" with "${drbd_master_resource}" INFINITY with-rsc-role=Master
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    ${SUDO}pcs  -f fs_cfg constraint order promote "${drbd_master_resource}" then start "${fs_resource}"
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    ${SUDO}pcs cluster cib-push --config fs_cfg
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    FILE_SYSTEM_RESOURCES+=("${fs_resource}")
    exit_code=$?;
    return ${exit_code}

}

check_drbd_uptodate(){
  get_drbd_disk_state
  for disk_state in "${DISK_STATE[@]}"; do
    if [ "$disk_state" != "UpToDate" ]; then
        echo "drbd status is not UpToDate/UpToDate"
        echo "please make sure the HA is configured and the drbd sync is completed"
        return 1
    fi
  done
  return 0
}

check_drbd_connected(){
  get_drbd_connectivity
  if [ "$DRBD_CONNECTIVITY" != "Connected" ]; then
    echo "drbd connectivity status '${DRBD_CONNECTIVITY}' is not valid for takover/failover operations."
    return 1
  fi
  return 0
}

start_pcs_resource(){
  get_drbd_resource_name
  exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
  get_drbd_disk_state
  if [ "${DISK_STATE[0]}" != "UpToDate" ]; then
    echo "please make sure the HA is configured and drbd on master node is UpToDate"
    return 1
  fi
  get_drbd_connectivity
  if [ "${DRBD_CONNECTIVITY}" == "SyncSource" ]; then
    echo "cannot start HA while drbd sync is running"
    return 1
  fi
  if [[ ! " ${VALID_DRBD_CONNECTIVITIES[@]} " =~ " ${DRBD_CONNECTIVITY} " ]]; then
    echo "drbd connectivity status '${DRBD_CONNECTIVITY}' is not valid to start HA"
    return 1
  fi
  ${SUDO}pcs resource cleanup  > /dev/null
  systemctl restart local-fs.target
  exit_code=$?
  if [ $exit_code -ne 0 ]; then
    echo "Failed to refresh disk paritions"
    return ${exit_code}
  fi
  _get_pcs_resources
  for resource in "${ALL_PCS_RESOURCES[@]}"; do
    echo "enabling resource ${resource}"
    ${SUDO}pcs resource enable ${resource}
  done
  return $?
}

stop_pcs_resource(){
  _get_pcs_resources
  for resource in "${ALL_PCS_RESOURCES[@]}"; do
    echo "disabling resource  ${resource}"
    ${SUDO}pcs resource disable ${resource}
  done
  return $?
}

ha_cleanup(){
    parse_ini "${HA_CONFIG_FILE}"
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    read_drbd_conf
    read_scripts_conf
    systemd_services=(${ini_config_arr[pacemaker_systemd_services]})
    for srv in "${systemd_services[@]}"; do
      echo "Check service ${srv} is stopped"
      ${SUDO}systemctl is-active ${srv} > /dev/null 2>&1
      if [ $? -eq 0 ]; then
        echo "service ${srv} is running. Stopping HA cluster before cleanup..."
        ${SUDO}ufm_ha_cluster stop
        break
      fi
    done

    # Creating Cleanup backup directory for DRBD_LOCATION files
    init_drbd_resource_backup "${CLEAN_UP_BACKUP_DIR}"
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "- ERROR - Failed to create DRBD backup dir" && return ${exit_code}

    ${SUDO}cp -pr ${DRBD_LOCATION} ${CLEAN_UP_BACKUP_DIR}/
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "- ERROR - Failed to backup DRBD data" && return ${exit_code}

    cleanup_pacemaker
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "-ERROR- Pacemaker cleanup failed"

    cleanup_drbd
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "-ERROR- DRBD cleanup failed"

    return 0
}

set_location_constraint() {
    local resource_name=$1
    local master=$2
    ${SUDO}pcs constraint location ${resource_name} prefers ${master}=${MASTER_SCORE}
    return $?
}

set_order_constraint() {
    local action=$1
    local first_resource=$2
    local second_resource=$3
    ${SUDO}pcs constraint order ${action} ${first_resource} then ${action} ${second_resource}
    return $?
}

set_cluster_props() {
    print_title "set cluster properties"
    ${SUDO}pcs property set no-quorum-policy=ignore
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    ${SUDO}pcs property set stonith-enabled=false
    exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
    ${SUDO}pcs resource defaults resource-stickiness=${STICKINESS}
    return $?
}

show_pcs_status() {
    ${SUDO}pcs status
    return $?
}

create_drbd_res_file() {
  local master_sync_ip="$1"
  local standby_sync_ip="$2"
  local ip_type="$3"
  echo "Creating drbd resource /tmp/${DRBD_RESOURCE}.res file"

  cat << EOF > /tmp/${DRBD_RESOURCE}.res
resource ${DRBD_RESOURCE} {
  protocol C;
  device ${DRBD_DEVICE};
  disk ${DRBD_DISK};
  meta-disk internal;
  floating ${ip_type} ${master_sync_ip}:${DRBD_PORT} {  }
  floating ${ip_type} ${standby_sync_ip}:${DRBD_PORT} {  }
  startup {
    wfc-timeout 10;
    degr-wfc-timeout 10;
  }
  disk {
    on-io-error detach;
  }
  net {
    timeout 60;
    connect-int	10;
    ping-int 10;
    after-sb-0pri discard-zero-changes;
    after-sb-1pri discard-secondary;
    after-sb-2pri disconnect;
  }
}
EOF
  return $?
}

create_drbd_resources() {
  echo "Creating drbd resources"
  local master_sync_ip=${LOCAL_SYNC_IP}
  local standby_sync_ip=${PEER_SYNC_IP}
  local ip_type="ipv4"
  if [ "${NODE_ROLE}" == "${NODE_ROLE_STANDBY}" ]; then
    master_sync_ip=${PEER_SYNC_IP}
    standby_sync_ip=${LOCAL_SYNC_IP}
  fi

  if [ "${PRIMARY_IP_TYPE}" == "${IP_TYPE_V6}" ]; then
    master_sync_ip="[${master_sync_ip}]"
    standby_sync_ip="[${standby_sync_ip}]"
    ip_type="ipv6"
  fi
  create_drbd_res_file "${master_sync_ip}" "${standby_sync_ip}" "${ip_type}"
  exit_code=$?
  if [ $exit_code -ne 0 ]; then
    echo "Failed to create drbd resources."
  fi
  return ${exit_code}
}

prepare_partition() {
  echo "Preparing drbd disk"
  init_drbd_resource_backup "${BACKUP_DIR}"
  unmount_partition "${DRBD_DISK}" "${DRBD_LOCATION}" "${BACKUP_DIR}"
}

restore_partition(){
  echo "mkfs.ext4 -F ${DRBD_DISK}"
  mkfs.ext4 -F ${DRBD_DISK}
  exit_code=$?

  if [[ "${MOUNT_DRBD_LOCATION}" != "no" ]]; then
    echo "mount ${DRBD_DISK} ${DRBD_LOCATION}"
    ${SUDO}mount ${DRBD_DISK} ${DRBD_LOCATION}
    exit_code=$?
  fi

  return $exit_code
}

setup_drbd() {

  # setup drbd resources
  check_multipathd_service
  copy_drbd_resource_file "${DRBD_RESOURCE}"
  exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
  prepare_partition
  exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
  load_drbd_module
  exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
  local primary=$(is_master_role)
  bringup_drbd_resources "${primary}" "${BACKUP_DIR}" "${DRBD_DISK}" "${DRBD_DEVICE}"
  exit_code=$?
  if [ $exit_code -ne 0 ]; then
    echo "Failed to setup drbd resources."
  fi
  return ${exit_code}
}

cleanup_drbd(){
  unmount_drbd
  exit_code=$?; [ ${exit_code} -ne 0 ] && echo "- ERROR - Failed to unmount DRBD disk" && return ${exit_code}

  bringdown_drbd_resources
  exit_code=$?; [ ${exit_code} -ne 0 ] && echo "- ERROR - Failed to shutdown DRBD resources" && return ${exit_code}

  stop_drbd_service
  exit_code=$?; [ ${exit_code} -ne 0 ] && echo "- ERROR - Failed to Stop DRBD service" && return ${exit_code}

  restore_drbd_global_conf
  exit_code=$?; [ ${exit_code} -ne 0 ] && echo "- ERROR - Failed to restore DRBD global conf" && return ${exit_code}

  remove_drbd_resource_files
  exit_code=$?; [ ${exit_code} -ne 0 ] && echo "- ERROR - Failed to remove DRBD resource files" && return ${exit_code}

  restore_partition
  exit_code=$?; [ ${exit_code} -ne 0 ] && echo "- ERROR - Failed to restore DRBD partition" && return ${exit_code}

  restore_ha_data_in_fstab ${DRBD_LOCATION} ${DRBD_DISK}
  exit_code=$?; [ ${exit_code} -ne 0 ] && echo "- ERROR - Failed to restore HA data in fstab" && return ${exit_code}

  return 0
}

read_drbd_conf() {
    DRBD_RESOURCE=${ini_config_arr[drbd_resource]}
    DRBD_LOCATION=${ini_config_arr[drbd_location]}
    DRBD_DISK=${ini_config_arr[drbd_disk]}
    DRBD_PORT=${ini_config_arr[drbd_port]}
    DRBD_DEVICE=${ini_config_arr[drbd_device]}

    if [[ "${1}" != "quiet" ]]; then
        echo "DRBD_RESOURCE:          [${DRBD_RESOURCE}]"
        echo "DRBD_LOCATION:          [${DRBD_LOCATION}]"
        echo "DRBD_PORT:              [${DRBD_PORT}]"
        echo "DRBD_DISK:              [${DRBD_DISK}]"
        echo "DRBD_DEVICE:            [${DRBD_DEVICE}]"
    fi
}

read_scripts_conf() {
    SCRIPT_PRE_START=${ini_config_arr[scripts_pre_start]}
    SCRIPT_PRE_CONFIG=${ini_config_arr[scripts_pre_config]}
    SCRIPT_POST_CONFIG=${ini_config_arr[scripts_post_config]}
    SCRIPT_POST_CLEANUP=${ini_config_arr[scripts_post_cleanup]}

    if [[ "${1}" != "quiet" ]]; then
        echo "SCRIPT_PRE_START:            [${SCRIPT_PRE_START}]"
        echo "SCRIPT_PRE_CONFIG:           [${SCRIPT_PRE_CONFIG}]"
        echo "SCRIPT_POST_CONFIG:          [${SCRIPT_POST_CONFIG}]"
        echo "SCRIPT_POST_CLEANUP:         [${SCRIPT_POST_CLEANUP}]"
    fi
}

read_general_conf() {
    GENERAL_SETTINGS_DIR=${ini_config_arr[general_settings_dir]}
    GENERAL_UFM_CONF=${ini_config_arr[general_ufm_conf]}

    if [[ "${1}" != "quiet" ]]; then
        echo "GENERAL_SETTINGS_DIR:            [${GENERAL_SETTINGS_DIR}]"
        echo "GENERAL_UFM_CONF:                [${GENERAL_UFM_CONF}]"
    fi
}

check_drbd_configured() {
  res=$(${SUDO}drbdadm sh-resources)
  if [[ -z ${res} ]]; then
    echo "false"
  else
    echo "true"
  fi
}

check_drbd_configuration_changed() {
  res_name=$1
  prev_drbd_conf_file=/etc/drbd.d/${res_name}.res
  new_drbd_conf_file=/tmp/${res_name}.res
  if diff -wB "$prev_drbd_conf_file" "$new_drbd_conf_file" &>/dev/null; then
    echo "false"
  else
    echo "true"
  fi
}

configure_drbd_service() {
  echo "configure drbd service"

  set_drbd_global_conf
  exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}

  create_drbd_resources
  exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}

  # Check if drbd is configured
  drbd_configured=$(check_drbd_configured)
  if [ ${drbd_configured} = 'true' ]; then
    # Compare new drbd configuration file with the old one and adjust if there changes.
    drbd_conf_changed=$(check_drbd_configuration_changed "$DRBD_RESOURCE")
    if [ ${drbd_conf_changed} = 'true' ]; then
      copy_drbd_resource_file "${DRBD_RESOURCE}"
      drbdadm adjust ha_data 
    else
      echo "drbd is already configured and no changes in drbd configuration."
    fi
  else 
    setup_drbd
  fi  
  return $?
}

is_vip_configured() {
    get_pcs_version
    if verlte ${PCS_VERSION} ${PCS_PRE_0_10_1_VERSION}; then
        ${SUDO}pcs resource show | grep -q ${VIRTUAL_IP_RSRC_NAME}
    else
        ${SUDO}pcs resource status | grep -q ${VIRTUAL_IP_RSRC_NAME}
    fi
    return $?
}

show_virtual_ip() {
    get_pcs_version
    if verlte ${PCS_VERSION} ${PCS_PRE_0_10_1_VERSION}; then
        vip_ip=`${SUDO}pcs resource show ${VIRTUAL_IP_RSRC_NAME} 2> /dev/null | grep ip= | awk -F= '{print $2}'`
    else
        vip_ip=`${SUDO}pcs resource config ${VIRTUAL_IP_RSRC_NAME} 2> /dev/null | grep ip= | awk -F= '{print $2}'`
    fi
    echo "VIRTUAL_IP:        ${vip_ip}"
}

is_master(){
    local result=false
    read_current_node_ip 'quiet'
    get_pcs_version
    local this_node=${CURRENT_NODE_IP}
    if [ -z ${CURRENT_NODE_IP} ]; then
      this_node=$(hostname)
    else
      if verlte ${PCS_VERSION} ${PCS_PRE_0_10_VERSION}; then
        this_node=$(get_hostname_from_ip ${CURRENT_NODE_IP})
        if [ -z "$this_node" ]; then
          this_node=$(uname -n)
        fi
      fi
    fi
    local master_node=$(get_master_from_cluster)
    if [[ "${this_node}" == "${master_node}"* ]]; then
      result=true
    fi
    echo ${result}
}

show_drbd_status() {
 get_drbd_resource_name
 exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
 get_drbd_disk_state
 get_drbd_role
 get_drbd_connectivity
 echo "DRBD_RESOURCE:     ${DRBD_RESOURCE}"
 echo "DRBD_CONNECTIVITY: ${DRBD_CONNECTIVITY}"
 echo "DISK_STATE:        ${DISK_STATE[0]}"
 echo "DRBD_ROLE:         ${DRBD_ROLE[0]}"
 echo "PEER_DISK_STATE:   ${DISK_STATE[1]}"
 echo "PEER_DRBD_ROLE:    ${DRBD_ROLE[1]}"
 echo "DRBD Sync Status:"
 cat /proc/drbd
}

IP_TYPE_V6="ipv6"
IP_TYPE_V4="ipv4"

get_drbd_resource_name() {
 DRBD_RESOURCE=$(${SUDO}drbdadm sh-resources)

 if [[ -z ${DRBD_RESOURCE} ]]; then
        echo "DRBD resource is not configured"
        return 1
 fi
 return 0
}

get_drbd_disk_state() {
    get_drbd_resource_name
    DISK_STATE=$(${SUDO}drbdadm dstate ${DRBD_RESOURCE} 2>/dev/null)
    exit_code=$?
    if [ $exit_code -ne 0 ]; then
        DISK_STATE="Unknown/Unknown"
    fi

    OFS=$IFS;IFS='/' read -r -a DISK_STATE <<< "$DISK_STATE";IFS=$OFS
}

get_drbd_role() {
    DRBD_ROLE=($(drbdadm status ${DRBD_RESOURCE} 2>/dev/null | grep "role:" | awk -F "role:" '{print $2}'))
    exit_code=$?
    if [ -z ${DRBD_ROLE} ]; then
        DRBD_ROLE=$(${SUDO}drbdadm role ${DRBD_RESOURCE} 2>/dev/null)
        exit_code=$?
        OFS=$IFS;IFS='/' read -r -a DRBD_ROLE <<< "$DRBD_ROLE";IFS=$OFS
    fi
    if [ $exit_code -ne 0 ]; then
        DRBD_ROLE=("Unknown" "Unknown")
    fi
}

get_drbd_connectivity() {
    DRBD_CONNECTIVITY=$(${SUDO}drbdadm cstate ${DRBD_RESOURCE} 2>/dev/null)
    exit_code=$?
    if [ $exit_code -ne 0 ]; then
        DRBD_CONNECTIVITY="Unknown"
    fi
}

validate_ip() {
  local ip_address="$1"
  local  result=1

  if [ -z "${ip_address}" ]; then
    return ${result}
  fi
  # try ipv4
  ip -4 route get ${ip_address} > /dev/null 2>&1
  result=$?
  if [ ${result} -eq 0 ]; then
    echo "${IP_TYPE_V4}"
    return ${result}
  fi

  # try ipv6
  ip -6 route get ${ip_address} > /dev/null 2>&1
  result=$?
  if [ ${result} -eq 0 ]; then
    echo "${IP_TYPE_V6}"
    return ${result}
  fi
  return ${result}
}


validate_virtual_ip() {
    echo "validate_virtual_ip"
    # validate virtual ip is either IPv4 or IPv6
    validate_ip $VIRTUAL_IP
    exit_code=$?
    if [[ ${exit_code} -ne 0 ]]; then
        echo "Not valid ip ${VIRTUAL_IP}"
        exit 1
    fi
    # validate virtual ip is not reserved
    if ping -c1 $VIRTUAL_IP &> /dev/null
    then
        echo "Virtual IP ${VIRTUAL_IP} is allocated, please choose free one."
        exit 1
    fi
}

verify_interface() {
  local if_name="$1"
  ip address show dev ${if_name} > /dev/null
  return $?
}

get_interface_ip() {
  local if_name="$1"
  local ip_type="$2"
  local scope=$3
  local result=
  local ip_addresses=
  if [ -z "${scope}" ]; then
    result=$(ip address show dev ${if_name} 2>/dev/null)
  else
    result=$(ip address show dev ${if_name} scope ${scope} 2>/dev/null)
  fi
  exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
  if [ "${ip_type}" == "${IP_TYPE_V4}" ]; then
    ip_addresses=$(echo "${result}" | awk '/inet / {split($2,a,"/"); print a[1]}')
  else
    ip_addresses=$(echo "${result}" | awk '/inet6 / {split($2,a,"/"); print a[1]}')
  fi
  ip_addresses=(${ip_addresses})
  ip_count=${#ip_addresses[@]}
  if [ ${ip_count} -gt 1 ]; then
    if [ -z "${scope}" ]; then
      get_interface_ip ${if_name} ${ip_type} "global"
    else
      echo ${ip_addresses[0]}
    fi
  else
    echo ${ip_addresses}
  fi
  return $?
}

get_peer_host_ip() {
  local host_name="$1"
  result=$(host ${host_name})
  exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
  echo "${result}" | egrep -o '([0-9]{1,3}\.){3}[0-9]{1,3}'
  return $?
}

get_hostname_from_ip() {
  local ip_addr="$1"
  result=$(host ${ip_addr})
  exit_code=$?; [ ${exit_code} -ne 0 ] && return ${exit_code}
  echo "${result}" | head -n1 | awk '{gsub(/.$/,""); print $NF}'
  return $?
}

get_local_host_ip() {
    hostname -i
    return $?
}

remove_ha_data_from_fstab() {
  drbd_location=$1
  sed -i "s|^UUID.*${drbd_location}|#&|" ${FSTAB_FILE}
}

restore_ha_data_in_fstab() {
  drbd_location=$1
  ha_data_disk=$2
  echo "Restore ${drbd_location} record in ${FSTAB_FILE}"
  ha_data_uuid=$(blkid -s UUID -o value ${ha_data_disk})
  if [ -z "${ha_data_uuid}" ]; then
    echo "${ha_data_uuid} UUId is empty"
    return 1
  fi
  sed -i "s|^.*\(UUID=[A-Fa-f0-9-]*\) \(.*${drbd_location}\)|UUID=${ha_data_uuid} \2|" ${FSTAB_FILE}
  return $?
}


check_service_is_active(){
    srv=$1
    status="active"
    systemctl is-active ${srv} > /dev/null 2>&1
    return $?
}

detach_standby(){
     local master_node=$(get_master_from_cluster)
     local slave_ip=$(pcs status pcsd | grep -v ${master_node} | grep -oE '[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}')
     if [ -z "$slave_ip" ]; then
        echo "can not find the slave ip node to detach it form cluster"
        return 1
     fi
     ${SUDO}pcs cluster node remove ${slave_ip} --force
     get_drbd_resource_name
     ${SUDO}drbdadm disconnect ${DRBD_RESOURCE}
     return $?
}

attach_standby(){
    create_drbd_resources
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "Failed to create drbd resource" && return ${exit_code}
    copy_drbd_resource_file "${DRBD_RESOURCE}"
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "Failed to copy drbd resource" && return ${exit_code}
    ${SUDO}drbdadm adjust "${DRBD_RESOURCE}"
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "Failed to adjust the drbd resourse" && return ${exit_code}
    get_pcs_version
    local peer_node=${PEER_SYNC_IP}
    if [ "${PRIMARY_IP_TYPE}" == "${IP_TYPE_V6}" ]; then
        peer_node=$(get_hostname_from_ip ${PEER_SYNC_IP})
    fi
    authenticate_cluster_nodes "${peer_node}"
    exit_code=$?; [ ${exit_code} -ne 0 ] && echo "Failed to authenticate the new node to the cluster" && return ${exit_code}
    if [[ -n ${PEER_SECONDARY_IP} ]]; then
        local peer="${peer_node},${PEER_SECONDARY_IP}"
    else 
        local peer="${peer_node}"
    fi
    ${SUDO}pcs cluster node add ${peer} --enable --start
    return $?
}

check_multipathd_service(){
    for multipathd_service in "${multipathd_services[@]}"; do
        check_service_is_active ${multipathd_service}
        if [ $? -eq 0 ]; then
            res=$(grep "blacklist" /etc/multipath.conf && echo true || echo false)
            if [[ ${res} != true ]]; then
                    echo "excluding devices  drbd0-drbd9 from multipathd  service configuration"
                    systemctl stop "${multipathd_services[@]}"
                    echo -e $multipath_conf >> /etc/multipath.conf
                    systemctl start "${multipathd_services[@]}"
                    break
            else
                res=$(grep "$multi_conf_flag" /etc/multipath.conf && echo true || echo false)
                if [[ ${res} != true ]]; then
                    echo "excluding devices  drbd0-drbd9 from multipathd  service configuration"
                    systemctl stop "${multipathd_services[@]}"
                    sed  -i 's:blacklist {:blacklist { \n devnode "^drbd[0-9]"\n:g' /etc/multipath.conf
                    systemctl start "${multipathd_services[@]}"
                    break

                fi
            fi
        fi
    done

}

clean_old_backup_dirs(){
    rm -rf ${BACKUP_DIR}
    rm -rf ${CLEAN_UP_BACKUP_DIR}
}

read_current_node_ip() {
    CURRENT_NODE_IP=${ini_config_arr[nodes_current_node_ip]}
    if [[ "${1}" != "quiet" ]]; then
        echo "CURRENT_NODE_IP:            [${CURRENT_NODE_IP}]"
    fi
}

enable_maintenance_mode(){
    maintenance_mode=$(check_maintenance_mode)
    if [[ "$maintenance_mode" = 'true' ]]; then
        echo "Maintenance mode is already enabled"
    else
        echo "Enable maintenance mode"
        pcs property set maintenance-mode=true
    fi
}

disable_maintenance_mode(){
    maintenance_mode=$(check_maintenance_mode)
    if [[ "$maintenance_mode" = 'false' ]]; then
        echo "Maintenance mode is already disabled"
    else
        echo "Disable maintenance mode"
        pcs property set maintenance-mode=false
    fi
}

check_maintenance_mode(){
    maintenance_mode=$(pcs property | grep maintenance-mode | awk '{print $NF}')
    echo "${maintenance_mode}"
}

check_local_ip_address() {
    ip_addr=$1
    is_local=$2
    if [ "$is_local" = true ]; then
        err_msg="Cannot find IP address ${ip_addr} on local machine."
        ip -oneline address show up | grep -q "${ip_addr}/"
        exit_code=$?; [ ${exit_code} -ne 0 ] && echo "Error: ${err_msg}" && exit 1
    else
        err_msg="The IP address ${ip_addr} is configured on the local machine."
        ip -oneline address show up | grep -q "${ip_addr}/"
        exit_code=$?; [ ${exit_code} -eq 0 ] && echo "Error: ${err_msg}" && exit 1
    fi
}

check_pacemaker_ip_connection() {
    ip_addr=$1
    echo "Check if IP address ${ip_addr} is listening to pacemaker's port number ${DEFAULT_PM_PORT}."
    nc -z ${ip_addr} ${DEFAULT_PM_PORT}
    return $?
}
