#!/bin/bash

SCRIPT_DIR="$(dirname "$(readlink -f "$0")")"
. ${SCRIPT_DIR}/.install_common.sh

QUIET_MODE=0
FORCE=0

uninstall(){
    # This indicator means when uninstalling UFM-HA
    # do not keep DRBD location mounted on the partition.
    # This use case is needed for UFM enterprise so that users
    # can uninstall UFM after removing UFM-HA.
    if [[ ! -f /etc/ufm-release  ]]; then
        export MOUNT_DRBD_LOCATION="no"
    fi

    /usr/local/bin/ufm_ha_cluster cleanup
    exit_code=$?
    if [ ${exit_code} -eq 1 ]; then
      echo "Cleanup of HA configurations Failed. Exiting..."
      exit 1
    fi

    rm_files

    echo "Uninstall finished!"
}


while getopts "qf" o; do
    case "${o}" in
        q)
            QUIET_MODE=1
            break
            ;;
    esac
done

if [[ -d ${UFM_HA_INSTALL_LOC} ]]; then 
    if [[ ${QUIET_MODE} -eq 1 ]]; then
        uninstall
    else
        read -p "Are you sure you want to uninstall UFM HA? [Y/n]" yn
        case $yn in
            [Yy]* ) uninstall;;
            [Nn]* ) exit;;
            * ) echo "Please answer yes or no."; exit 1;;
        esac
    fi
else 
    echo "UFM HA is not installed."
fi
